### Nested Logit model estimation

install.packages("maxLik")
library(maxLik)

### データファイルの読み込み
Data <- read.csv("ensyu.csv",header=T)
## データ数:Dataの行数を数える
hh <- nrow(Data)

## パラメータの初期値の設定
b0 <- numeric(7) 


##### Logit model の対数尤度関数の定義 #####

fr <- function(x) {
  ### パラメータの宣言
  ## 定数
  b1 <- x[1]
  b2 <- x[2]
  b3 <- x[3]
  b4 <- x[4]

  ## 目的地までの所要時間
  d1 <- x[5]
  
  ## 料金
  f1 <- x[6]
   
  ##スケールパラメータ
  mu <- x[7]
  
  

  ## 対数尤度のための変数を宣言
  LL = 0

  ### 今回用いる交通手段は以下5つ
  ## 鉄道(train)  - (公共)
  ## バス(bus)    - (公共)
  ## 自動車(car)  - (私有)
  ## 自転車(bike) - (私有)
  ## 徒歩(walk)   - (私有)
  
  ## 効用の計算:説明変数にした行列を入れる
                                         # 時間                     # 料金                    # 定数項
  train  <- Data$代替手段生成可否train*exp(d1*Data$総所要時間train/100 +f1*Data$費用train/100 + b1*matrix(1,nrow =hh,ncol=1))
  bus    <- Data$代替手段生成可否bus  *exp(d1*Data$総所要時間bus/100   +f1*Data$費用bus/100   + b2*matrix(1,nrow =hh,ncol=1))
  car    <- Data$代替手段生成可否car  *exp(d1*Data$所要時間car/100                        + b3*matrix(1,nrow =hh,ncol=1))
  bike   <- Data$代替手段生成可否bike *exp(d1*Data$所要時間bike/100                       + b4*matrix(1,nrow =hh,ncol=1))
  walk   <- Data$代替手段生成可否walk *exp(d1*Data$所要時間walk/100                                                     )

  ### 選択確率の計算
  
  ###上位ネスト構造：公共・私有の選択
  ## ログサム変数の設定
  logsum.public  <- log( ( (train+bus)!=0) * (train + bus) + ((train+bus)==0))
  logsum.private <- log( ( (car + bike + walk)!=0) * (car + bike + walk) + ((car + bike + walk)==0))

  ## ログサム変数を用いて公共・私有の選択確率P(LV1)を表現
  nume.public   <- exp(mu*logsum.public)
  nume.private  <- exp(mu*logsum.private)

  deno <- nume.public + nume.private
  P.public  <- nume.public  / deno
  P.private <- nume.private / deno

  
  ###下位ネスト構造：交通機関の選択
  ###公共・私有のなかでの選択：条件付き確率P(LV2|LV1)
  
  ##公共ネストにおける交通機関の選択確率
  deno.public     <- train + bus
  P.train.public  <- train / ((deno.public!=0)*deno.public + (deno.public==0))
  P.bus.public    <- bus   / ((deno.public!=0)*deno.public + (deno.public==0))

  ##私有ネストにおける交通機関の選択確率
  deno.private    <- car + bike + walk
  P.car.private   <- car   / ((deno.private!=0)*deno.private + (deno.private==0))
  P.bike.private  <- bike  / ((deno.private!=0)*deno.private + (deno.private==0))
  P.walk.private  <- walk  / ((deno.private!=0)*deno.private + (deno.private==0))

  ### 最後に公共・私有それぞれの交通手段の同時確率を計算： P(LV1, LV2) = P(LV2|LV1)*P(LV1)
  P.train <- P.train.public * P.public
  P.bus   <- P.bus.public   * P.public
  P.car   <- P.car.private  * P.private
  P.bike  <- P.bike.private * P.private
  P.walk  <- P.walk.private * P.private

  ## 選択確率0になってしまった場合に起こる問題の回避：真数条件に反するエラーをケア
  P.train <- (P.train!=0)*P.train + (P.train==0)
  P.bus   <- (P.bus!=0)  *P.bus   + (P.bus==0)
  P.car   <- (P.car!=0)  *P.car   + (P.car==0)
  P.bike  <- (P.bike!=0) *P.bike  + (P.bike==0)
  P.walk  <- (P.walk!=0) *P.walk  + (P.walk==0)

  ## 選択結果
  C.train   <- Data$代表交通手段 =="鉄道"
  C.bus     <- Data$代表交通手段 =="バス"
  C.car     <- Data$代表交通手段 =="自動車"
  C.bike    <- Data$代表交通手段 =="自転車"
  C.walk    <- Data$代表交通手段 =="徒歩"

  ### 対数尤度の計算
  LL <- colSums(C.train*log(P.train) + C.bus*log(P.bus) +
                C.car*log(P.car)   + C.bike*log(P.bike) +C.walk*log(P.walk))

}

##### 対数尤度関数frの最大化#####

##パラメータ値の最適化
res <- maxLik(fr,start=b0,method="BFGS")
res
## パラメータ推定値、ヘッセ行列
b <- res$estimate
hhh <- res$hessian

## t値の計算
tval <- b/sqrt(-diag(solve(hhh)))

## 初期尤度
L0 <- fr(b0)
## 最終尤度
LL <- res$maximum


##### 結果の出力 #####
print(res)
## 初期尤度
print(L0)
## 最終尤度
print(LL)
##ρ^2値
print((L0-LL)/L0)
## 修正済ρ^2値
print((L0-(LL-length(b)))/L0)
##パラメータ推定値 
print(b)
## t値 
print(tval)
